Package: pizzarr
Type: Package
Title: Slice into 'Zarr' Arrays
Version: 0.2.0
Authors@R: c(
    person(given = "David",
           family = "Blodgett",
           role = c("cre", "aut"),
           email = "dblodgett@usgs.gov",
           comment = c(ORCID = "0000-0001-9489-1710")),
    person(
      given = "Mark",
      family = "Keller",
      email = "mark_keller@g.harvard.edu",
      role = c("aut"),
      comment = c(ORCID = "0000-0003-3003-874X")
    ),
    person(
      given = "Artür", 
      family = "Manukyan", 
      email = "artur-man@hotmail.com",
      role = c("aut"), 
      comment = c(ORCID = "0000-0002-0441-9517")),
    person(given = "zarr-developers", role = "cph")
  )
Description: An implementation of chunked, compressed,
  N-dimensional arrays for R. 'Zarr' spec V2 (2024)
  <doi:10.5281/zenodo.11320255>.
Language: en-US
License: MIT + file LICENSE
BugReports: https://github.com/zarr-developers/pizzarr/issues
URL: https://zarr.dev/pizzarr/,
        https://github.com/zarr-developers/pizzarr
Depends: R (>= 4.2)
Imports: jsonlite, stats, R6, qs2, stringr, memoise, utils
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Suggests: testthat, knitr, covr, bslib, pkgdown, rmarkdown, crul,
        blosc, vcr (>= 0.6.0), bench, bit64, withr
Config/testthat/parallel: false
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-20 12:57:51 UTC; dblodgett
Author: David Blodgett [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-9489-1710>),
  Mark Keller [aut] (ORCID: <https://orcid.org/0000-0003-3003-874X>),
  Artür Manukyan [aut] (ORCID: <https://orcid.org/0000-0002-0441-9517>),
  zarr-developers [cph]
Maintainer: David Blodgett <dblodgett@usgs.gov>
Repository: CRAN
Date/Publication: 2026-04-20 16:50:02 UTC
Built: R 4.6.0; ; 2026-04-28 08:27:57 UTC; unix
