Note: these files have been copied from https://github.com/manzt/zarrita.js/tree/4acc746/fixtures


```sh
cd inst/extdata/fixtures
conda env create -f environment.yml
conda activate pizzarr-fixtures

cd v2
python ./generate-v2.py
```