# v2 store keys
ARRAY_META_KEY <- '.zarray'
GROUP_META_KEY <- '.zgroup'
ATTRS_KEY <- '.zattrs'

# v3 store key
# V3 uses a single zarr.json file per node, replacing .zarray/.zgroup/.zattrs.
ZARR_JSON <- 'zarr.json'
