# Reference: https://github.com/gzuidhof/zarr.js/blob/292804/src/rawArray/index.ts

#' The Zarr RawArray class.
#' @title RawArray Class
#' @docType class
#' @description
#' TODO: Add export annotation / remove keywords internal annotation once implemented.
#' @rdname RawArray
#' @keywords internal
RawArray <- R6::R6Class("RawArray",
  public = list(
    #' @description
    #' Create a new RawArray instance.
    #' @return A `RawArray` instance.
    initialize = function() {
      # TODO
    }
  )
)
