% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{zarrs_compiled_features}
\alias{zarrs_compiled_features}
\title{Return compiled zarrs feature flags as a character vector.}
\usage{
zarrs_compiled_features()
}
\description{
Called once at \code{.onLoad} to populate \code{.pizzarr_env$zarrs_available}.
The feature list is determined at compile time via \verb{cfg!} checks.
Also installs a no-op panic hook on first call.
}
