% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zarrs-dispatch.R
\name{selection_to_ranges}
\alias{selection_to_ranges}
\title{Convert pizzarr indexer to zarrs ranges}
\usage{
selection_to_ranges(indexer)
}
\arguments{
\item{indexer}{A BasicIndexer with SliceDimIndexer or IntDimIndexer dims.}
}
\value{
List of integer vectors.
}
\description{
Convert pizzarr indexer to zarrs ranges
}
\keyword{internal}
