% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zarrs-bridge.R
\name{pizzarr_upgrade}
\alias{pizzarr_upgrade}
\title{Upgrade to the zarrs backend}
\usage{
pizzarr_upgrade()
}
\description{
Prints the command to install pizzarr from r-universe with the
compiled zarrs backend, or reports that zarrs is already available.
}
