% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{zarr_volcano}
\alias{zarr_volcano}
\title{Create a demo Zarr group containing R's volcano dataset}
\usage{
zarr_volcano()
}
\value{
A \code{ZarrGroup} containing a single array called \code{"volcano"}.
}
\description{
Writes the \code{\link[datasets]{volcano}} matrix into a temporary
DirectoryStore as a Zarr array named \code{"volcano"} and returns the
opened group.
}
\examples{
g <- zarr_volcano()
v <- g$get_item("volcano")
image(v$get_item("...")$data, main = "Maunga Whau Volcano")
}
