% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\name{decode_fill_value_v3}
\alias{decode_fill_value_v3}
\title{Encode a fill_value for V3 zarr.json.
V3 requires special float values as JSON strings: NaN, Infinity, -Infinity.
Normal numeric values are returned as unboxed scalars.}
\usage{
decode_fill_value_v3(fill_value)
}
\arguments{
\item{fill_value}{The fill value to encode.}
}
\value{
A value suitable for inclusion in a jsonlite::toJSON call.
}
\description{
Encode a fill_value for V3 zarr.json.
V3 requires special float values as JSON strings: NaN, Infinity, -Infinity.
Normal numeric values are returned as unboxed scalars.
}
\keyword{internal}
