% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{has_bit64}
\alias{has_bit64}
\title{Check if the bit64 package is available}
\usage{
has_bit64()
}
\value{
Logical.
}
\description{
Check if the bit64 package is available
}
\keyword{internal}
