% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zarrs-bridge.R
\name{is_zarrs_available}
\alias{is_zarrs_available}
\title{Check whether the zarrs Rust backend is available}
\usage{
is_zarrs_available()
}
\value{
Logical scalar. \code{TRUE} when the package was installed with the
compiled zarrs backend (r-universe tier), \code{FALSE} on the CRAN tier
(pure R).
}
\description{
Check whether the zarrs Rust backend is available
}
\keyword{internal}
