% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zarrs-dispatch.R
\name{can_use_zarrs_create}
\alias{can_use_zarrs_create}
\title{Check whether the zarrs create path can handle this array}
\usage{
can_use_zarrs_create(store, dtype)
}
\arguments{
\item{store}{A Store object.}

\item{dtype}{A Dtype object.}
}
\value{
Logical scalar.
}
\description{
Returns TRUE when zarrs is available, the store is writable and
has a store identifier, and the dtype is numeric/logical (not
object or string).
}
\keyword{internal}
