% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/creation.R
\name{zarr_open}
\alias{zarr_open}
\title{Convenience function to open a group or array using file-mode-like semantics.}
\usage{
zarr_open(store = NA, mode = NA, path = NA, ...)
}
\arguments{
\item{store}{: Store
A mapping that supports string keys and bytes-like values.}

\item{mode}{: \code{'r', 'r+', 'a', 'w', 'w-'}, optional
Persistence mode: 'r' means read only (must exist); 'r+' means
read/write (must exist); 'a' means read/write (create if doesn't
exist); 'w' means create (overwrite if exists); 'w-' means create
(fail if exists).}

\item{path}{: string, bytes, optional
Path under which array is stored.}

\item{...}{Additional arguments to pass to zarr_open_array or zarr_open_group.}
}
\value{
ZarrArray or ZarrGroup
}
\description{
Convenience function to open a group or array using file-mode-like semantics.
}
