% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atomic.R
\name{is_integer}
\alias{is_integer}
\title{Check if a value is an integer R vector or scalar.}
\usage{
is_integer(s)
}
\description{
Check if a value is an integer R vector or scalar.
}
\keyword{internal}
